; 
;* Description:	Heizgriffsteuerung mit AT90S1200
;*              10 LED als Display an PD3 bis PD6 und PB0 bis PB5
;*                  0..10 Stufen, Blinkend
;*              zwei Taster zum Ein-Aus-Schalten und programmieren an PD2 und PD1
;*              PowerFET an PD0
;* Version:		1.0
;* First writing:	02-09-2006
;* Last Updated:	03-09-2006
;* Target:		AT 90s1200 
;* Author:		Martin Schneider
;***************************************************************************

.nolist				;Suppress listing of include file
.include "C:\Programme\Atmel\AVR Tools\AvrAssembler2\Appnotes\1200def.inc"		
.list

;***** Global register variables
.def	save_sreg	= r0
.def	iwr0		= r1

.def	key_ct0		= r2
.def	key_ct1		= r3
.def	key_state 	= r4
.def	key_release	= r5
.def	key_press	= r16
.def	key_rpt		= r21
.def	key_long	= r22

.def	time_cnt	= r6
.def	rpt_cnt		= r24

.def	wr0		= r17
.def	wr1		= R18	;General use working register

.def	blink_flag	= R19	;Timeout value passed to subroutine
.def	state  		= R20	;

.def	time0		= r7
.def	pwm0		= r8
.def	long_cnt	= r23

.def	pwm_cnt		= r25
.def	pwm_val		= r26

;EEPROM related variables
.def     EEAdr  =r25         ;temporary storage of address
.def     EEdrd_s =r9         ;result data byte
.def	  EEdwr_s =r29		;data to write

;***** Other equates
.equ	crystal = 5000000       ; 5.000MHz crystal

.equ	key_port 	= pind
.equ	SwInp		= PD1	;Schwarze Taste, 
.equ	RtInp		= PD2	;Rote Taste, auch einschalten
.equ	REP_MSK		= (1<<SwInp)  ; nur Schwarze Taste
.equ	REP_START	= 50	; 500 ms
.equ	REP_NEXT	= 20	; 200 ms

.equ	pwm_port	= portb
.equ	pwm_out 	= PB6	;PWM Output, Gate vom PowerFET
.equ	PWM_CYCLE	= 10	; PWM Auflsung
.equ	BLINK_FREQ	= 50	; Blinkperiode (2*10ms)

.equ	led_l_port	= portd
.equ	LED1		= PD0
.equ	LED2		= PD3
.equ	LED3		= PD4
.equ	LED4		= PD5
.equ	LED5		= PD6
.equ	LED_L_MSK	= (1<<LED1 | 1<<LED2 | 1<<LED3 | 1<<LED4 | 1<<LED5)	; PD0, PD1, PD4, PD5, PD6

.equ	led_h_port	= portb
.equ	LED6		= PB0
.equ	LED7		= PB1
.equ	LED8		= PB2
.equ	LED9		= PB3
.equ	LED10		= PB4
.equ	LED_H_MSK	= (1<<LED6 | 1<<LED7 | 1<<LED8 | 1<<LED9 | 1<<LED10)	; PB0..PB4

; States of the main FSM
.equ	IDLE	= 0; IDLE
.equ	START	= 1; StartUp heating
.equ	TIME	= 2; Measure Time for StartUp Heating
.equ	HEAT	= 3;	normal heating
.equ	DOWN	= 4; wait for shutting down to enter sleep mode

;	Timer/Counter prescaler values
.equ	EEMWE	=2
.equ	TCK16	=5		;Timer/Counter runs from CK / 1024 ~ 5 kHz
.equ	TREL0	=207		;Timer/Counter reload value -> ca. 10ms Overflow
;.equ	TREL0	=244		;Timer/Counter reload value -> ca. 2,5ms Overflow

;EEPROM related variables
.equ    EEStart = 4




; IRQ vector table
.org 0x00
	rjmp	reset		; RESET: reset
	reti		    	; INT0: Nur zum Aufwachen!
	rjmp	timer		; T0OVF: timer 0 overflow
	reti			; ANA_Comp

;	Main program entry point on reset

reset:
        cli			; disable interrupts
	ldi	wr1,0b00010000	;low level on int0 causes wakeup interrupt from sleep
	out	MCUCR,wr1	;
        
	ldi	wr1,TREL0	;load Timer0 reload value
	out	TCNT0,wr1	;
	ldi	wr1,TCK16	;load Timer0 clock select
	out	TCCR0,wr1	;Start timer
	ldi	wr1,(1<<TOIE0)	;Enable Timer 0 interrupt
	out	TIMSK,wr1

	
	ldi	wr1,0xbf	;Turn off all outputs
	out	PORTB,wr1
	ldi	wr1,(LED_H_MSK+(1<<pwm_out))	;Make port B pin all outputs
	out	DDRB,wr1
	
	ldi	wr1,0xff	;Turn off outputs, activate pull-ups
	out	PORTD,wr1
	ldi	wr1,LED_L_MSK	;Make port D pin 1,2 inputs, others outputs
	out	DDRD,wr1

	sbi 	ACSR,ACD	;shut down comparator to save power

	ldi	state,1<<IDLE		; start in state IDLE

       
    ldi	pwm_cnt, PWM_CYCLE
    ser	blink_flag	
	clr	time_cnt
	ldi	long_cnt, BLINK_FREQ

	clr pwm_val
	clr	key_press		; Tastenbits lschen
	clr	key_release
 	clr key_rpt

	rjmp	enter_sleep	; nach power-on erstmal schlafen

;=============================================================
; restore timer and PWM from EEPROM
;
wake_up:
	ldi	EEAdr,EEStart		;Clear the temporary address  
	out	EEAR,EEAdr	;Move the temporary address to EEAR 	
    rcall	EERead_seq     ;Get the data from EEPROM to time0
    mov	time0,EEdrd_s
    rcall	EERead_seq     ;Get the data from EEPROM to pwm_val
    mov	pwm0,EEdrd_s

	ldi	wr1, (PWM_CYCLE+1)
	cp	pwm0, wr1	; fehlerhafter Wert?
	brlt	EEP_ok	; nein: alles o.k.
	ldi	wr1, (PWM_CYCLE/2)	; falscher Wert: lade 50% als Standard
	mov	pwm0, wr1
EEP_ok:	

	ldi	EEAdr,0		;Clear the temporary address  
	out	EEAR,EEAdr	;Move the temporary address to EEAR 	

;===============================================================
; main loop, checks only the keys
;===============================================================
keys:
    sbrc	state, IDLE	; berspringe wenn nicht im State IDLE
	rjmp	fsm_idle
    sbrc	state, START	; berspringe wenn nicht im State START
	rjmp	fsm_start
    sbrc	state, TIME	; berspringe wenn nicht im State TIME
	rjmp	fsm_time
    sbrc	state, HEAT	; berspringe wenn nicht im State HEAT
	rjmp	fsm_heat
    sbrc	state, DOWN	; berspringe wenn nicht im State DOWN
	rjmp	fsm_down
keys_ready:
	rjmp	led_output	; 
	; from led_output will return to keys:
;---------------------------------------------------------------

fsm_idle:
	sbrc 	key_press, RtInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_idle_key		;Rote Taste behandeln
	rjmp	fsm_finish	; zurck
fsm_idle_key:
 	cbr 	key_press,1<<RtInp ;Tastenbit1 lschen, Job wird erledigt,
	sbrc 	key_state, SwInp  ;berspringe wenn Schwarze Taste nicht gedrckt
	rjmp	fsm_idle_check		; goto Zeit messen
	; normales Einschalten
    mov 	time_cnt, time0	; Aufheizzeit setzen
	ldi		pwm_val, 11	; volle Pulle heizen, LED blinken
    lsl		state		; Bit schieben auf START
 	rjmp 	fsm_finish         ;fertig...
fsm_idle_check:
 	cbr 	key_press,1<<SwInp ;Tastenbit1 lschen, Job wird erledigt,
 	cbr 	key_rpt,(1<<RtInp)+(1<<SwInp) ;Tastenbits lschen, Job wird erledigt,
	; Zeit messen
    ldi 	wr0, 0xff	; Aufheizzeit auf max setzen
	mov		time_cnt, wr0
	ldi		pwm_val, 11	; volle Pulle heizen, LED blinken
    lsl		state		; Bit schieben auf START
	lsl		state 		; Bit schieben auf TIME
  	rjmp 	fsm_finish         ;fertig...

fsm_start:
	sbrc 	key_press, RtInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_start_ready		;Rote Taste behandeln
	sbrc 	key_press, SwInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_start_key		;Schwarze Taste behandeln
	tst 	time_cnt		  ;teste Timer
	breq	fsm_start_ready		;springe wenn abgelaufen
	rjmp	fsm_finish	; zurck
fsm_start_key:
	tst		key_release
	breq	fsm_start_key
fsm_start_ready:
 	clr 	key_press ;Tastenbits lschen, Job wird erledigt,
 	clr 	key_rpt
	clr		key_release
	mov		pwm_val, pwm0	; vorgemerkt heizen, Stufe anzeigen
	lsl		state 		; Bit schieben auf TIME
	lsl		state 		; Bit schieben auf HEAT
 	rjmp 	fsm_finish         ;fertig...
	
fsm_time:
	sbrc 	key_press, RtInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_time_key		;Rote Taste behandeln
	sbrc 	key_press, SwInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_time_key		;Schwarze Taste behandeln
	tst 	time_cnt		  ;teste Timer
	breq	fsm_time_ready		;springe wenn abgelaufen
	rjmp	fsm_finish	; zurck
fsm_time_key:
	tst		key_release
	breq	fsm_time_key
fsm_time_ready:
 	clr 	key_press ;Tastenbits lschen, Job wird erledigt,
 	clr 	key_rpt
	clr		key_release
	mov		time0, time_cnt	; Restzeit holen
	com 	time0			; und negieren -> neuer Zeitwert
	mov		pwm_val, pwm0	; vorgemerkt heizen, Stufe anzeigen
	lsl		state 		; Bit schieben auf HEAT
 	rjmp 	fsm_finish         ;fertig...
	
fsm_heat:
	tst		key_press		; weiter erst bei gedrckter Taste
	breq	fsm_finish
	sbrc 	key_release, RtInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_heat_up		;Rote Taste behandeln
	sbrc 	key_rpt, SwInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_heat_finish		;Schwarze Taste lang behandeln
	sbrc 	key_release, SwInp  ;berspringe, wenn Bit gelscht
	rjmp	fsm_heat_down		;Schwarze Taste behandeln
	rjmp	fsm_finish	; zurck
fsm_heat_up:
 	clr 	key_press ;Tastenbits lschen, Job wird erledigt,
	clr		key_release
	ldi		wr0,10
	cp		pwm0, wr0	; already max?
	breq	fsm_finish	; yes, ready
	; no: increment pwm level
	inc		pwm0
	mov		pwm_val, pwm0	; vorgemerkt heizen, Stufe anzeigen
 	rjmp 	fsm_finish         ;fertig...
fsm_heat_down:
 	clr 	key_press ;Tastenbits lschen, Job wird erledigt,
	clr		key_release
	tst		pwm0		; already 0?
	breq	fsm_finish	; yes: ready
	; no: decrement pwm level
	dec		pwm0
	mov		pwm_val, pwm0	; vorgemerkt heizen, Stufe anzeigen
	rjmp 	fsm_finish         ;fertig...
fsm_heat_finish:
 	clr 	key_press ;Tastenbits lschen, Job wird erledigt,
 	clr 	key_rpt
	clr		key_release
	lsl		state 		; Bit schieben auf DOWN
fsm_finish:
 	rjmp 	keys_ready         ;fertig...
	
fsm_down:
	ldi		state, START	; restart with START
;	inc		pwm0			; Heizstufe wieder eines hoch (Ausgleich )
;	mov		pwm_val, pwm0	; vorgemerkt heizen, Stufe anzeigen
	rjmp	store			; save all things in EEPROM and go to sleep

;=============================================================
; led output 
; activate the need number of LEDS regarding to pwm_val
; if the number > 10, the activate all and blinking
; activate: LOW_Active!!!
;=============================================================
led_output:
	tst		pwm_val		; all off?
	breq	led_off
    cpi		pwm_val, 6
        ; less than 6 LED?
    brsh	led_high	; no: goto set_high
        ; yes: only low part
led_low:
        ; set (deactivate) high part
    cli		; to prevent IRQ from disturbing
    in		wr0, led_h_port
    sbr		wr0, LED_H_MSK
    out		led_h_port, wr0
	sei
    ; set low part
    in		wr0, led_l_port
    sbr		wr0, LED_L_MSK
    out		led_l_port, wr0
    ; clear the needed low LED
    ldi		wr0, 1		; one LED?
	cp		wr0, pwm_val
	breq	led_1
    inc		wr0		; two LED?
	cp		wr0, pwm_val
	breq	led_2
    inc		wr0		; three LED?
	cp		wr0, pwm_val
	breq	led_3
    inc		wr0		; four LED?
	cp		wr0, pwm_val
	breq	led_4
	; five LED!
	cbi		led_l_port, LED5	; clear bit for LED5
led_4:
	cbi		led_l_port, LED4	; clear bit for LED4
led_3:
	cbi		led_l_port, LED3	; clear bit for LED3
led_2:
	cbi		led_l_port, LED2	; clear bit for LED2
led_1:
	cbi		led_l_port, LED1	; clear bit for LED1


    rjmp	led_ready

led_off:
    ; set low part
    in		wr0, led_l_port
    sbr		wr0, LED_L_MSK
    out		led_l_port, wr0
     ; set high part
    cli		; to prevent IRQ from disturbing
    in		wr0, led_h_port
    sbr		wr0, LED_H_MSK
    out		led_h_port, wr0
	sei
        
led_ready:
	rjmp	keys

        
led_high:
    cpi		pwm_val, 11
    ; less than 11 LED?
    brsh	led_blink	; no: activate all and blink
    ; yes:
    ; clear (activate)  low part
    in		wr0, led_l_port
    cbr		wr0, LED_L_MSK
    out		led_l_port, wr0
        ; set all high LED
    cli		; to prevent IRQ from disturbing
    in		wr0, led_h_port
    sbr		wr0, LED_H_MSK
    out		led_h_port, wr0
	sei
    ; clear the needed high LED
    ldi		wr0, 6		; calculate the mask for upper LED

	cp		wr0, pwm_val
	breq	led_6
    inc		wr0		; seven LED?
	cp		wr0, pwm_val
	breq	led_7
    inc		wr0		; eight LED?
	cp		wr0, pwm_val
	breq	led_8
    inc		wr0		; nine LED?
	cp		wr0, pwm_val
	breq	led_9
	; ten LED!
	cbi		led_h_port, LED10	; clear bit for LED10
led_9:
	cbi		led_h_port, LED9	; clear bit for LED9
led_8:
	cbi		led_h_port, LED8	; clear bit for LED8
led_7:
	cbi		led_h_port, LED7	; clear bit for LED7
led_6:
	cbi		led_h_port, LED6	; clear bit for LED6
        
    rjmp	led_ready

led_blink:
    tst		blink_flag	; blink_falg auf 0?
    breq	led_off		; yes: switch LED off
    ; no: switch LED on
    ; clear low part
    in		wr0, led_l_port
    cbr		wr0, LED_L_MSK
    out		led_l_port, wr0
	; clear high part
    cli		; to prevent IRQ from disturbing
    in		wr0, led_h_port
    cbr		wr0, LED_H_MSK
    out		led_h_port, wr0
	sei
    rjmp	led_ready

;=============================================================
; Timer ISR 
; Tasten entprellen, PWM Wert ausgeben
; und Zeitmessung
;=============================================================
timer:
	; Statusregister sichern
	in		save_sreg, SREG
	;load Timer0 reload value
	ldi		wr1,TREL0
	out		TCNT0,wr1
	; Tasten einlesen:
;i = key_state ^ (~console);   // key changed?
	in		iwr0,key_port  ;Port D einlesen
	com		iwr0
	eor		iwr0,key_state  ;
;ct0 = ~(ct0 & i);        // reset or count ct0
	and		key_ct0,iwr0  ;
	com		key_ct0    ;count
;ct1 = ct0 ^ ct1 & i;        // reset or count ct1
	and		key_ct1,iwr0  ;
	eor		key_ct1,key_ct0  ;
;i &= ct0 & ct1;          // count until roll over?
	and		iwr0,  key_ct0  ;input AND last counterstate
	and		iwr0,  key_ct1  ;
;// **** Release Erkennung
;key_release |= key_state & i; // 1->0: key release detect
;// ****
    mov		wr1,iwr0
	and		wr1, key_state
	or		key_release, wr1
;key_state ^= i;          // then toggle debounced state
	eor		key_state,iwr0  ;toggle state
;key_press |= key_state & i;   // 0->1: key press detect
	and		iwr0,key_state  ;0-1-transition
	or		key_press,iwr0  ;store key_press detect





    ldi		wr1,REP_MSK	; repeat mask prfen
	mov		iwr0, wr1
	and		iwr0, key_state ;
	brne	key_rpt1
	ldi		rpt_cnt, REP_START
key_rpt1:
	dec		rpt_cnt
	brne	key_deb_ready
	ldi		rpt_cnt, REP_NEXT
	or		key_rpt, iwr0
key_deb_ready:
	; ready with debouncing, count long_time 
count_long:
	dec		long_cnt
	brne	count_ready
	; null durchgang: Toggle BlinkFlag and go on
	ldi		long_cnt, BLINK_FREQ
	com		blink_flag
	; Langzeit timer: max. 255*0.5s (2min)
	tst		time_cnt	; schon abgelaufen?
	breq	count_ready	; ja: fertig
	dec		time_cnt	; nein: runterzhlen
count_ready:	
	; PWM ausgabe: 0 - aus
	;             1..9 - PWM Stufe
	;             10 an
	;             > 10 an + LED blinken
pwm_handling:
	; count PWM down
	dec		pwm_cnt
        ; schon bei 0?
	brne	pwm_high	; nein: weiter bei Wertabfrage	
		; ja: PWM_counter neu laden
	ldi		pwm_cnt, PWM_CYCLE	; zehnstufige PWM
		; Ausgang auf low stellen, wenn PWM_Val <= 10;
    cp		pwm_val, pwm_cnt
	brge	pwm_ready	
	cbi		pwm_port,pwm_out

pwm_high:
	cp		pwm_val,pwm_cnt	; PWM wert schon erreicht?
	brlt	pwm_ready	; nein: weitermachen
	; ja: Ausgang auf high stellen
pwm_set:
	sbi		pwm_port,pwm_out
pwm_ready:
	out		SREG, save_sreg
    reti

;***************************************************************************
;* 
;* EERead_seq
;*
;* This subroutine increments the address stored in EEAR and reads the 
;* EEPROM into the register variable "EEdrd_s".
;*
;*
;***************************************************************************


EERead_seq:
	in 		EEAdr,EEAR       ;get address
	out 	EEAR,EEAdr      ;output address
	sbi 	EECR,EERE        ;set EEPROM Read strobe
		;This instruction takes 4 clock cycles since
		;it halts the CPU for two clock cycles
	sbi 	EECR,EERE        ;set EEPROM Read strobe 2nd time
		;This instruction takes 4 clock cycles since
		;it halts the CPU for two clock cycles
	in 		EEdrd_s,EEDR      ;get data
	in 		EEAdr,EEAR       ;get address
    inc 	EEAdr           ;increment address
    out 	EEAR,EEAdr      ;store it 
	ret

;==============================================================
;***************************************************************************
;* 
;* EEWrite_seq
;*
;* This subroutine increments the EEPROM address by one and waits until the 
;* EEPROM is ready for programming. It then programs the EEPROM with 
;* register variable "EEdwr_s".

;* Number of words	:1200 ; 7 + return
;*			:8515 ; 10 + return
;* Number of cycles	:1200 ; 10 + return (if EEPROM is ready)
;*			:8515 ; 15 + return (if EEPROM is ready)
;* Low Registers used	:None
;* High Registers used:	:3 (EEdwr_s,EEwtmp,EEwtmph)
;*
;***************************************************************************

EEWrite_seq:

; Write sequence for 90S1200

	sbic	EECR,EEWE	;if EEWE not clear
	rjmp	EEWrite_seq	;    wait more

	out		EEAR,EEAdr	
	out		EEDR,EEdwr_s	;output data
	sbi 	EECR,EEMWE	;set master write enable
	sbi		EECR,EEWE	;set EEPROM Write strobe
				;This instruction takes 4 clock cycles since
				;it halts the CPU for two clock cycles
 	inc		EEAdr		;increment address 8515

	ret

;=======================================================================
;    Store to EEprom routine and go to sleep afterwards
;=======================================================================
store:
    ; action!
	cli			; stop further irq
    ; switch off the LED display
	in		wr0, led_h_port
    sbr		wr0, LED_H_MSK
    out		led_h_port, wr0
    ; set low part
    in		wr0, led_l_port
    sbr		wr0, LED_L_MSK
    out		led_l_port, wr0
	; PWM aus
	cbi		pwm_port,pwm_out

        ; store counter into eeprom	
	ldi		EEAdr,EEStart
	mov		eedwr_s,time0
	rcall	eewrite_seq	
	mov		eedwr_s,pwm0
	rcall	eewrite_seq

	nop
	nop
	nop

enter_sleep:
;	ldi		EEAdr, 0		; reset the Adresspointer
;	out 	EEAR,EEAdr
	
	; switch off the MCU
	ldi		wr1,0x30	;turn on sleep mode (Power down mode) 
	out		MCUCR,wr1	;interrupt on low level.
	ldi		wr1,0x40	;enable external interrupts
	out		GIMSK,wr1
	sei			;enable global interrupts ready	

farewell:
	sleep			;go to sleep mode
; if power: go on working
	cli			; stop further irq

 	cbr 	key_press,1<<RtInp ;Tastenbit1 lschen, Job wird erledigt,
 	cbr 	key_rpt,(1<<RtInp) ;Tastenbits lschen, Job wird erledigt,
 	cbr 	key_press,1<<SwInp ;Tastenbit1 lschen, Job wird erledigt,
 	cbr 	key_rpt,(1<<SwInp) ;Tastenbits lschen, Job wird erledigt,

	ldi		wr1,0b00010000	; low level on int0 causes interrupt
	out		MCUCR,wr1	;
	ldi		wr1,0x00	;disable external interrupts
	out		GIMSK,wr1

	;load Timer0 reload value
	ldi		wr1,TREL0
	out		TCNT0,wr1

	sei			;enable global interrupts ready	


    rjmp    wake_up		; restart working


;*********************************************************************
;**** EEPROM Data
;****
;*********************************************************************
         .eseg

;EmptyPos:.db 00  ;leave eeprom location 00 empty due to bug
;in AVR that randomly erases the contens
InitString1:
	.db 0x30,0x30,0x30,0x30 ; dummy, empty
	.db	0x0a		; 5 sec startzeit
	.db	0x03		; 30% dauerleistung
	.db 0x39, 0x39, 0x39, 0x39 ; dummy, end-of-data
